package org.apidesign.apifest08.test;

import static org.apidesign.apifest08.currency.ConvertorFactory.createConvertor;
import static org.apidesign.apifest08.currency.ConvertorFactory.mergeConvertors;
import static org.apidesign.apifest08.currency.MoneyImpl.money;
import static org.apidesign.apifest08.test.Task1Test.CZK;
import static org.apidesign.apifest08.test.Task1Test.SKK;
import static org.apidesign.apifest08.test.Task1Test.USD;
import junit.framework.TestCase;

import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ExtendedConvertor;

/** There are many currencies around the world and many banks manipulate
 * with more than one or two at the same time. As banks are usually the
 * best paying clients, which is true even in case of your Convertor API,
 * it is reasonable to listen to their requests.
 * <p>
 * The quest for today is to enhance your existing convertor API to hold
 * information about many currencies and allow conversions between any of them.
 * Also, as conversion rates for diferent currencies usually arise from various
 * bank departments, there is another important need. There is a need to
 * compose two convertors into one by merging all the information about
 * currencies they know about.
 */
public class Task2Test extends TestCase {
    public Task2Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    // As in Task1Test, keep in mind, that there are three parts
    // of the whole system:
    // 1. there is someone who knows the current exchange rate
    // 2. there is someone who wants to do the conversion
    // 3. there is the API between 1. and 2. which allows them to communicate
    // 
    // Please backward compatibly enhance your existing API to support following
    // usecases:
    //
    
    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 75 CZK. This is method for the group of users that
     *  knows the exchange rate, and needs to use the API to create objects
     *  with the exchange rate. Anyone shall be ready to call this method without
     *  any other method being called previously. The API itself shall know
     *  nothing about any rates, before this method is called.
     */
    public static Convertor createTripleConvertor() {
        // Rates: 1USD = 15CZK
        // Rates: 1USD = 20SKK
        // Rates: 75CZK = 100SKK
	   return mergeConvertors(
			   createConvertor(money(1, USD),  money(15, CZK)),
			   createConvertor(money(1, USD),  money(20, SKK)),
			   createConvertor(money(75, CZK), money(100, SKK))
       );
    }

    /** Define convertor that understands three currencies. Use it.
     */
    public void testConvertorForUSDandCZKandSKK() throws Exception {
        Convertor c = createTripleConvertor();

        // convert $5 to CZK using c:
        assertEquals("Result is 75 CZK", money(75, CZK),c.convert(money(5,USD), CZK));

        // convert $5 to SKK using c:
        assertEquals("Result is 100 SKK", money(100, SKK),c.convert(money(5,USD), SKK));

        // convert 200SKK to CZK using c:
        assertEquals("Result is 150 CZK", money(150, CZK),c.convert(money(200,SKK), CZK));

        // convert 200SKK to USK using c:
        assertEquals("Result is 10 USD", money(10, USD),c.convert(money(200,SKK), USD));
    }

    /** Merge all currency rates of convertor 1 with convertor 2.
     * Implement this using your API, preferably this method just delegates
     * into some API method which does the actual work, without requiring
     * API clients to code anything complex.
     */
    public static Convertor merge(Convertor one, Convertor two) {
        return mergeConvertors((ExtendedConvertor)one, (ExtendedConvertor)two);
    }

    /** Join the convertors from previous task, Task1Test and show that it
     * can be used to do reasonable conversions.
     */
    public void testConvertorComposition() throws Exception {
        Convertor c = merge(
            Task1Test.createCZKtoUSD(),
            Task1Test.createSKKtoCZK()
        );

        // convert $5 to CZK using c:
        assertEquals("Result is 85 CZK", money(85, CZK),c.convert(money(5,USD), CZK));

        // convert $8 to CZK using c:
        assertEquals("Result is 136 CZK", money(136, CZK),c.convert(money(8,USD), CZK));

        // convert 1003CZK to USD using c:
        assertEquals("Result is 59 USD", money(59, USD),c.convert(money(1003,CZK), USD));

        // convert 16CZK using c:
        assertEquals("Result is 20 SKK", money(20, SKK),c.convert(money(16,CZK), SKK));

        // convert 500SKK to CZK using c:
        assertEquals("Result is 400 CZK", money(400, CZK),c.convert(money(500,SKK), CZK));

    }
}
